#!/usr/bin/env python
# -*- coding: utf-8 -*-
###########################################################################
# HamsterPidgin
#
# Author:  J. Félix Ontañón 2008 <fontanon at emergya dot es>
# Comment: Updates Pidgin with Hamster current activity using
#	   hamster dbus interface. Based on AmarokPidgin by:
#          Keegan Carruthers-Smith 2008 <keegan.csmith@gmail.com>
# Use: 	   Execute as is while d-bus enhaced hamster and pidgin are running
# License: Distributed under the GPLv2
###########################################################################

import dbus
import re
import signal
import gobject
from sys import stdin, exit
from ConfigParser import ConfigParser
from StringIO import StringIO

DEFAULT_CONFIG = """
[HamsterPidgin]
status_name = Hamster
status_message = Working on to %activity [Hamster]
display = status # Where to display activity. either status or nick
"""

class HamsterPidgin(object):

    def __init__(self):
        """
        Connects too Pidgin's dbus interface.
        Stores the following variables:
            default - The default status
            purple - Pidgin dbus interface
            status - The status id for AmarokPidgin
	    get_hamster_activity - Hamster's DBus method for get the current activity
            activity - The activity string for the status. If not set, it is null
        """

        self.config = None
        self.parse_config()

        # Get the purple object
        bus = dbus.SessionBus()
        obj = bus.get_object("im.pidgin.purple.PurpleService", "/im/pidgin/purple/PurpleObject")
        purple = dbus.Interface(obj, "im.pidgin.purple.PurpleInterface")

 	# Get the hamster object
	obj = bus.get_object("org.hamster", "/org/hamster")
	get_hamster_activity = obj.get_dbus_method("get_activity")

        # Get the status object
        status_name = self.config.get("HamsterPidgin", "status_name")
        status = purple.PurpleSavedstatusFind(status_name)

        if status == 0: # Doesn't exist, create
            current = purple.PurpleSavedstatusGetCurrent()
            status_type = purple.PurpleSavedstatusGetType(current)
            status = purple.PurpleSavedstatusNew(status_name, status_type)

        self.default        	= purple.PurpleSavedstatusGetCurrent()
        self.purple         	= purple
        self.status         	= status
        self.nicks          	= {}
        self.nicks_in_use   	= set()
	self.get_hamster_activity = get_hamster_activity
        self.revert_status  	= False

        self.activity = self.get_current_activity()
        self.update_display(self.activity)


    def parse_config(self):
        "Reads in the configuration file."
        config = ConfigParser()

        # Set Defaults
        defaults = StringIO(DEFAULT_CONFIG)
        config.readfp(defaults)
        defaults.close()

        # Read in config
        try:
            config_fp = file("HamsterPidgin.ini", "r")
            config.readfp(config_fp)
            config_fp.close()
        except:
            print "Could not read HamsterPidgin.ini"

        self.config = config


    def _update_status(self, message):
        "Changes the displayed status"

        # Update the status object if necessary
        current = self.purple.PurpleSavedstatusGetCurrent()
        if current != self.status:
            # Removing code adds functionality! :D
            #status_name = self.purple.PurpleSavedstatusGetTitle(self.status)
            #status_type = self.purple.PurpleSavedstatusGetType(current);
            #self.purple.PurpleSavedstatusDelete(status_name)
            #self.status = self.purple.PurpleSavedstatusNew(status_name, status_type)
            self.default = current
            if self.revert_status: # Switch back to Hamster Status
                current = self.status

        # Update Purple's status
        self.purple.PurpleSavedstatusSetMessage(self.status, message)
        if current == self.status:
            self.purple.PurpleSavedstatusActivate(self.status)


    def _update_nick(self, message):
        "Changes the displayed nickname"

        nick_ids = self.purple.PurpleAccountsGetAllActive()

        for nick_id in nick_ids:
            # Store default nick
            if nick_id not in self.nicks:
                nick = self.purple.PurpleAccountGetAlias(nick_id)
                self.nicks[nick_id] = nick

            # Update nick
            self.purple.PurpleAccountSetAlias(nick_id, message)

            # Add to nicks in use, for checking when not in use
            self.nicks_in_use.add(nick_id)

        # Restore nicks
        for nick_id in self.nicks_in_use.difference(nick_ids):
            self.purple.PurpleAccountSetAlias(nick_id, nick[nick_id])


    def decode(self, message):
        "Tries to decode the message"
        encodings = ['utf8', 'iso-8859-1']
        
        try:
            import chardet
            encodings.append(chardet.detect(message)['encoding'])
        except ImportError:
            pass

        # Try and decode message
        for encoding in encodings:
            try:
                message = message.decode(encoding)
            except:
                self.log("DecodeError: Could not decode '%s' as %s" % \
                         (message, encoding))
            else:
                return message
        
        return message.decode('utf8', 'replace')


    def update_display(self, message):
        "Changes the displayed message."

        message = self.decode(message)
        if not message:
            return

        # Update the necessary display
        display = self.config.get("HamsterPidgin", "display")

        # Either nick or status. Defaults to status
        if display != 'nick':
            display = 'status'
            self._update_status(message)
        else:
            self._update_nick(message)


    def restore_nicks(self):
        "Restores the nicks to there defaults"

        for nick_id in self.nicks:
            self.purple.PurpleAccountSetAlias(nick_id, self.nicks[nick_id])


    def get_current_activity(self):
        new_status = self.config.get("HamsterPidgin", "status_message")

        value = self.get_hamster_activity()
        new_status = new_status.replace("%activity", value)

        return new_status

# Made a global for cleanup script
hamsterPidgin = None

def cleanup(signum, frame):
    "Tries too change the status back too the default"
    global hamsterPidgin
    try:
        if hamsterPidgin:
            hamsterPidgin.purple.PurpleSavedstatusActivate(hamsterPidgin.default)
            hamsterPidgin.restore_nicks()
            hamsterPidgin = None
    except:
        pass

    if signum == signal.SIGTERM:
        exit(0)

if __name__ == "__main__":
    from dbus.mainloop.glib import DBusGMainLoop
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    bus = dbus.SessionBus()

    signal.signal(signal.SIGTERM, cleanup)
    try:
        hamsterPidgin = HamsterPidgin()
    except dbus.DBusException:
	cleanup(0,0)
	exit(1)

    bus.add_signal_receiver(hamsterPidgin.update_display, dbus_interface="org.hamster", signal_name="update_activity")
    gobject.MainLoop().run()

    exit(0)
